import java.io.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

public class EventOrder extends DefaultHandler
{
  public static void main(String argv[])
  {
    if (argv.length != 1) {
      System.err.println("Usage: java EventOrder filename");
      System.exit(1);
    }

    // Use an instance of ourselves as the SAX event handler
    DefaultHandler handler = new EventOrder();
    // Use the default (non-validating) parser
    SAXParserFactory factory = SAXParserFactory.newInstance();
    System.out.println("validating => " + factory.isValidating() + "..\n");

    try {
      SAXParser saxParser = factory.newSAXParser();
      saxParser.parse(new File(argv [0]), handler);
    } catch (Throwable t) {
      t.printStackTrace();
    }
  }

  /* Data members */
  /* --------------- */
  private int eventNum = 0;
  private Locator locator;


  /* Handler methods */
  /* --------------- */

  public void setDocumentLocator (Locator locator) {
    this.locator = locator;
    eventNum++;
    System.out.println("Event #" + eventNum + ": setDocumentLocator called.");
  }

  public void startDocument()
    throws SAXException
  {
    eventNum++;
    System.out.println("Event #" + eventNum + ": startDocument called.");
  } // startDocument

  public void endDocument()
    throws SAXException
  {
    eventNum++;
    System.out.println("Event #" + eventNum + ": endDocument called.");
  } // endDocument

  public void startElement(String namespaceURI,
      String lName, // local name
      String qName, // qualified name
      Attributes attrs)
    throws SAXException
  {
    String eName = lName; // element name
    if ("".equals(eName)) eName = qName; // namespaceAware = false
    eventNum++;
    System.out.println("Event #" + eventNum + ": startElement for <" + eName + "> called."

          + "It ends at line " +
         locator.getLineNumber() + " column " +
         (locator.getColumnNumber()+1) + ".");
  } // startElement

  public void endElement(String namespaceURI,
      String lName, // simple name
      String qName) // qualified name
    throws SAXException
  {
    String eName = lName; // element name
    if ("".equals(eName)) eName = qName; // namespaceAware = false
eventNum++;
    System.out.println("Event #" + eventNum + ": endElement for </" + eName + "> called.");
  } // endElement

  public void characters(char[] ch,
      int start,
      int length)
    throws SAXException
  {
    eventNum++;
    System.out.println("Event #" + eventNum + ": characters called on character string of length "
      + length + ": \"" + new String(ch, start, length) + "\".");
  } // characters

  public void ignorableWhitespace(char[] ch,
      int start,
      int length)
    throws SAXException
  {
    eventNum++;
    System.out.println("Event #" + eventNum + ": ignorableWhitespace called with " + length + " white spaces.");
  } // ignorableWhitespace

  public void processingInstruction(java.lang.String target,
      java.lang.String data)
    throws SAXException
  {
    eventNum++;
    System.out.println("Event #" + eventNum + ": processingInstruction called on instruction: ["
      + target + "|" + data + "].");
  } // processingInstruction

  public void skippedEntity(java.lang.String name)
    throws SAXException
  {
    eventNum++;
    System.out.println("Event #" + eventNum + ": skippedEntity       called on skipped entity: " + name + ".");
  }

  public void startPrefixMapping(java.lang.String prefix,
      java.lang.String uri)
    throws SAXException
  {
    eventNum++;
    System.out.println("Event #" + eventNum + ": startPrefixMapping called on the prefix to namespace mapping: ["
    + prefix + " => " + uri + "].");
  } // startPrefixMapping

  public void endPrefixMapping(java.lang.String prefix)
    throws SAXException
  {
    eventNum++;
    System.out.println("Event #" + eventNum + ": endPrefixMapping called on the prefix "
      + prefix + ".");
  } // endPrefixMapping
} // EventOrder